#!/bin/bash
# /opt/squeezeplay/bin/install_ap.sh

MYTEMPFILE=/tmp/install.ap
HOSTAPDCFG=/etc/hostapd/hostapd.conf
INTERFACES=/etc/network/interfaces
MYSSID=
MYPASS=
MYPERF=
MYPERN=

function write_i () {
WLANMAC=`ifconfig -a | grep "wlan0" 2>&1 | awk -F\addr {'print $2'} | awk -F\  {'print $1'}`
WLANMAC=02"${WLANMAC: -15}"
echo -n "# This file describes the network interfaces available on your system
# and how to activate them. For more information, see interfaces(5).

# The loopback network interface
auto lo
    iface lo inet loopback
 
# The primary network interface
#auto eth0
    #iface eth0 inet dhcp

# The secondary network interface
#auto wlan0
    #iface wlan0 inet dhcp

auto br0
iface br0 inet dhcp
    #network 192.168.1.0
    #broadcast 192.168.1.255
    #gateway 192.168.1.254
    #netmask 255.255.255.0
    hwaddress ether $WLANMAC
    bridge-ports eth0 wlan0
    up /sbin/ifconfig br0 up 
    post-up /bin/sleep 10 ; /usr/sbin/hostapd -B /etc/hostapd/hostapd.conf 
    pre-down /usr/bin/killall hostapd
    down /sbin/ifconfig br0 down
">$MYTEMPFILE
}

function write_g () {
echo "ctrl_interface=/var/run/hostapd
###############################
# Basic Config
###############################
macaddr_acl=0
auth_algs=1
# Most modern wireless drivers in the kernel need driver=nl80211
driver=nl80211
##########################
# Local configuration...
##########################
interface=wlan0
bridge=br0
hw_mode=g
channel=1
ssid=$MYSSID
macaddr_acl=0
auth_algs=1
ignore_broadcast_ssid=0
wpa=2
wpa_passphrase=$MYPASS
wpa_key_mgmt=WPA-PSK
wpa_pairwise=TKIP
rsn_pairwise=CCMP">$MYTEMPFILE
}

function write_n () {
#  details see http://w1.fi/cgit/hostap/plain/hostapd/hostapd.conf
echo "#  /etc/hostapd/hostapd.conf
#  authoritative information and specifics are available here:
#  http://hostap.epitest.fi/gitweb/gitweb.cgi?p=hostap.git;a=blob_plain;f=hostapd/hostapd.conf
#  https://wiki.debianforum.de/WLAN-Access-Point_mit_hostapd_und_USB-Stick
#
ctrl_interface=/var/run/hostapd
###############################
#  Basic Config
###############################
macaddr_acl=0
auth_algs=3

#  Most modern wireless drivers in the kernel need driver=nl80211
#  WiFi B/G/N based on the Ralink RT2870 chipset (connected to the internal USB port).
driver=nl80211

##########################
#  Local configuration...
##########################
#  if you want to bridge the onboard eth0 and the wireless USB adapter's wlan0, this should work
interface=wlan0
bridge=br0

#  yes, it says 802.11g, but the n-speeds get layered on top of it
hw_mode=g

#  this enables the 802.11n speeds and capabilities
#  ...  You will also need to enable WMM for full HT functionality.
ieee80211n=1
wmm_enabled=1
wmm_ac_be_aifs=3
wmm_ac_be_cwmin=4
wmm_ac_be_cwmax=10
wmm_ac_be_txop_limit=0
wmm_ac_be_acm=0

#  adjust to fit your location
#  Here is the authoritative list of countries:
#  http://www.iso.org/iso/country_codes/iso_3166_code_lists/country_names_and_code_elements.htm
country_code=DE

#  let your AP broadcast the settings that agree with the above-mentioned regulatory requirements per country
ieee80211d=1

#  makes the SSID visible and broadcasted
ignore_broadcast_ssid=0

#  adjust to fit your preference
ssid=$MYSSID

#  this is how I set mine up
auth_algs=3
wpa=2
wpa_passphrase=$MYPASS
wpa_key_mgmt=WPA-PSK
wpa_pairwise=TKIP
rsn_pairwise=CCMP
macaddr_acl=0

#  self-explanatory, but not all channels may be enabled for you - check /var/log/messages for details
#  also, make sure you survey your 2.4GHz environment, and find some uncluttered/uncrowded channels
# freq		HT40-		HT40+
# 2.4 GHz	5-13		1-9
# 5 GHz		40,48,56,64	36,44,52,60
channel=1
#  these have to be set in agreement w/ channel and some other values... read hostapd.conf docs
#  ...  these can be tuned to specific capabilities per device/chipset
#
#ht_capab=[GF][SMPS-STATIC][HT40-][SHORT-GI-20][SHORT-GI-40][RX-STBC1][MAX-AMSDU-3839]
ht_capab=[HT40+][SHORT-GI-40][SHORT-GI-20]

#  IIUC, this enables reauthentication for clients, so the passphrase doesn't have to be re-entered
#eap_reauth_period=3600
#  ... my understanding... 3600 seconds = 1 hr, so your device would have to reauthenticate if it rejoined after 1 hr
#  I have mine set quite a bit higher than 3600 seconds... so that my iPhone doesn't ask me to type in the password 
#  frequently...  

#   debugging output - uncomment them to activate; issue  hostapd -d /etc/hostapd/hostapd.conf
#   to get debugging info in visible/real-time form
#logger_syslog=-1
#logger_syslog_level=2
#logger_stdout=-1
#logger_stdout_level=2">$MYTEMPFILE
}

if [ -f $HOSTAPDCFG ]; then
	MYSSID=`cat $HOSTAPDCFG | grep -oP "^ssid=\K.*"`
	MYPASS=`cat $HOSTAPDCFG | grep -oP "^wpa_passphrase=\K.*"`
	MYPERF=`cat $HOSTAPDCFG | grep -oP "^ieee80211n=\K.*"`
	MYPERF=${MYPERF:-0}
fi
while true; do
	echo 
	read -e -p "Enter SSID: "								-i "${MYSSID:-myhotspot}"	MYSSID
	read -e -p "Enter passphrase: "							-i "${MYPASS:-qwerty0987}"	MYPASS
	read -e -p "Use 802.11n performance(0 - no ,1 - yes): "	-i "${MYPERF:-1}"			MYPERN
	if [ "$MYSSID" != "" ] && [ "$MYPASS" != "" ]; then break; fi
done
if [ ! -f "/etc/default/hostapd" ]; then
	echo
	echo installing packages
	sudo apt-get -y update
	sudo apt-get -y install hostapd
	sudo apt-get -y install bridge-utils
	sudo mv /etc/init.d/hostapd /
	echo updating /etc/default/hostapd
	sudo sed 's/^DAEMON_CONF=.*/DAEMON_CONF="\/etc\/hostapd\/hostapd.conf"/I' -i /etc/default/hostapd
fi

if [ ! -f $HOSTAPDCFG ] || [ "$MYPERN"  != "$MYPERF" ] ; then
	MYPERF=$MYPERN
	if [ $MYPERF != 1 ]; then 
		echo creating 802.11g $HOSTAPDCFG
		write_g 'MYTEMPFILE'
	else
		echo creating 802.11n $HOSTAPDCFG
		write_n 'MYTEMPFILE'
	fi
	sudo mv $MYTEMPFILE		$HOSTAPDCFG
	sudo chown root:root	$HOSTAPDCFG
	sudo chmod 644			$HOSTAPDCFG
else
	echo updating $HOSTAPDCFG
fi
echo access point conifguration
sudo sed "s/^ssid=.*/ssid=$MYSSID/I" -i $HOSTAPDCFG
sudo sed "s/^wpa_passphrase=.*/wpa_passphrase=$MYPASS/I" -i $HOSTAPDCFG

echo creating $INTERFACES
write_i 'MYTEMPFILE'
sudo mv $MYTEMPFILE		$INTERFACES
sudo chown root:root	$INTERFACES
sudo chmod 644			$INTERFACES

#sudo nano $HOSTAPDCFG
#sudo nano $INTERFACES
#ifconfig -a
